package com.tagfriendly;

import com.tagfriendly.jdbc.CacheConnector;
import com.tagfriendly.freedb.*;
import com.tagfriendly.relational.RelationalLoader;
import com.tagfriendly.solr.Builder;

import java.sql.Connection;
import java.sql.SQLException;
import java.io.File;
import java.util.List;

/**
 * Created by: gary at Oct 23, 2007 10:23:58 PM
 * %Persistent.%BuildIndices 
 */
public class Main {
    private static final String FREEDB_LOAD = "FreeDbLoad";
    private static final String RELATIONAL_CONVERT = "RelationalConvert";
    private static final String INDEX_BUILD = "IndexBuild";

    private static final int MAX = Integer.parseInt(System.getProperty("MAX","-1"));

    public static void main(String args[]) {
        if (args == null || args.length == 0) {
            System.err.println("Nothing to do. Quitting.");
            System.exit(0);
        }

        String action = args[0];
        String host = args[1];
        int port = Integer.parseInt(args[2]);
        String db = args[3];
        String user = args[4];
        String pass = args[5];

        if (action.equals(FREEDB_LOAD))
            FreeDbLoad(host,port,db,user,pass,args[6],Integer.parseInt(args[7]));
        else if (action.equals(RELATIONAL_CONVERT))
            RelationalConversion(host,port,db,user,pass);
        else if (action.equals(INDEX_BUILD))
            IndexBuild(host,port,db,user,pass,Integer.parseInt(args[6]));
        
    }

    private static void IndexBuild(String host, int port, String db, String user, String pass, int workers) {
        try {
            Connection[] con = new Connection[workers];
            for (int i = 0; i < con.length; i++)
                con[i] = CacheConnector.connect(host,port,db,user,pass);
            Builder builder = new Builder(con);
            builder.start();
        } catch (Throwable th) {
            th.printStackTrace();
        }
    }

    private static void RelationalConversion(String host, int port, String db, String user, String pass) {
        try {
            final Connection con = CacheConnector.connect(host,port,db,user,pass);
            RelationalLoader loader = new RelationalLoader(con);
            System.out.println("Building entry list.");
            List<Integer> entries = loader.getEntriesToProcess();
            System.out.println("Entry list build. Proceeding...");
            for (int entryId : entries) {
                try {
                    System.out.print("Loading entry " + entryId + "...");
                    loader.load(entryId);
                    System.out.println(" done.");
                } catch (SQLException ex) {
                    ex.printStackTrace();
                }
            }
        } catch (Throwable th) {
            th.printStackTrace();
        }
    }

    private static void FreeDbLoad(String host, int port, String db, String user, String pass, String file, final int minQuality) {

        try {
            final long start = System.currentTimeMillis();
            final Connection con = CacheConnector.connect(host,port,db,user,pass);
            final FreeDbReader reader = new FreeDbReader(new File(file),1000);
            reader.start();

            new Thread("Taker") {
                public void run() {
                    int count = 0;
                    final int max = MAX <= 0 ? Integer.MAX_VALUE : MAX;
                    try {
                        FreeDbLoader loader = new FreeDbLoader(con);
                        FreeDbEntry entry = reader.next();
                        while (entry != null && count < max) {
                            if (entry.quality() >= minQuality) {
                                loader.load(entry);
                                System.out.println(++count +
    //                                    " " + entry.getArchivePath() +
                                        " " + entry.getTitle() +
                                        "  ..." + entry.quality() + ", " +
    //                                    entry.getManyUnicode() + "," + entry.getManyQ() +
                                        ", " + entry.qualityExplanation());
                            }
                            entry = reader.next();
                        }
                    } catch (SQLException ex) {
                        ex.printStackTrace();
                    } catch (LoadException ex) {
                        ex.printStackTrace();
                    } catch (Throwable th) {
                        th.printStackTrace();
                    }
                    long end = System.currentTimeMillis();
                    long time = (end-start) / 1000;
                    System.out.println("Took " + time + " seconds for " + count + " entries.");
                    System.exit(0);
                }
            }.start();
        } catch (Throwable th) {
            th.printStackTrace();
        }
    }
}
