package com.tagfriendly.common;

import org.apache.log4j.Logger;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.io.*;
import java.util.List;
import java.util.ArrayList;

/**
 * Created by: gary at Oct 26, 2007 2:03:33 PM
 */
public abstract class TableCreation {
    private static final Logger logger = Logger.getLogger(TableCreation.class);

    private final Connection con;

    public TableCreation(Connection con) {
        this.con = con;
    }

    public abstract String getSchema();
    public abstract SqlFileDescription[] getTableDescriptions();

    public final void makeIndices() throws SQLException {
        run(getSchema(),new SqlFileDescription(getSchema() + ".indices",true));
    }

    public final void makeTables() throws SQLException {
        SqlFileDescription[] descriptions = getTableDescriptions();

        ArrayList<SqlFileDescription> resourceArr = new ArrayList<SqlFileDescription>();
        List<String> tables = getTableNames(getSchema());
        for (SqlFileDescription desc : descriptions)
            if (!tables.contains(desc.getResource()))
                resourceArr.add(desc);

        run (getSchema(),resourceArr.toArray(new SqlFileDescription[resourceArr.size()]));
    }

    private void run(String schema, SqlFileDescription... descr) {
        for (int i = 0; i < descr.length; i++)
            loadDdl(descr[i].getResource(),descr[i].getByLine());
    }

    private void loadDdl(String ddl, boolean byLine) {
        try {
            InputStream in = ResourceLoader.getResource(getClass().getClassLoader(),"FreeDb/resources" + File.separator + ddl);
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            StringBuilder sb = new StringBuilder();
            String line = reader.readLine();
            while (line != null) {
                if (byLine) {
                    try {
                        int sz = con.createStatement().executeUpdate(line);
                        logger.info("Ran ddl: " + line);
                    } catch (SQLException ex) {
                        logger.error(line,ex);
                    }
                }
                sb.append(line);
                line = reader.readLine();
            }
            if (!byLine) {
                try {
                    int sz = con.createStatement().executeUpdate(sb.toString());
                    logger.info("Ran ddl: " + ddl);
                } catch (SQLException ex) {
                    logger.error(line,ex);
                }
            }
        } catch (IOException ex) {
            logger.error(ex);
        }
    }

    private List<String> getTableNames(String schema) throws SQLException {
        ResultSet tables = con.getMetaData().getTables(null,schema,null,null);
//        ResultSetMetaData tableInfo = tables.getMetaData();
        List<String> tableNames = new ArrayList<String>();
        while (tables.next())
            if ("TABLE".equals(tables.getString("TABLE_TYPE")))
                tableNames.add(tables.getString("TABLE_SCHEM").toLowerCase() + "." + tables.getString("TABLE_NAME").toLowerCase());
        return tableNames;
    }

    private static void dump(ResultSet rs, ResultSetMetaData meta) throws SQLException {
        for (int i = 0; i < meta.getColumnCount(); i++) {
            try {
                System.out.println(meta.getColumnName(i) + "=" + rs.getString(i) + ", " + meta.getColumnClassName(i));
            } catch (Throwable th) {
                System.out.println(i + ": " + th.getMessage());
            }
        }
    }
}
