package com.tagfriendly.freedb;

import com.tagfriendly.freedb.FreeDbEntry;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Created by: gary at Oct 23, 2007 10:07:50 PM
 */
public class FreeDbLoader {
    private final Connection con;
    private final PreparedStatement newEntry;
    private final PreparedStatement fetchMaxId;
    private final PreparedStatement newTitle;

    private final Lock insLock = new ReentrantLock();

    public FreeDbLoader(Connection con) throws SQLException {
        new FreeDbTableCreation(con).makeTables();
        new FreeDbTableCreation(con).makeIndices();
        this.con = con;
        newEntry = con.prepareStatement("insert into freedb.entry(discLength,revision,submitted,title,year,extd,genre,playOrder,discId,album,artist,archivePath,quality,qualityExplanation,enc,manyUnicode,manyQ,visited) " +
            "values(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)");
        fetchMaxId = con.prepareStatement("select max(entryId) from freedb.entry");
        newTitle = con.prepareStatement("insert into freedb.title(trackOrder,name,extt,entryId) values (?,?,?,?)");
    }

    public void load(FreeDbEntry entry) throws LoadException {
        insLock.lock();
        try {
            // create the entry.
            newEntry.clearParameters();
            int quality = entry.quality(); // ensures this gets called. sets transient data.
            newEntry.setInt(1,entry.getDiscLength());
            newEntry.setInt(2,entry.getRevision());
            newEntry.setString(3,entry.getSubmitted());
            newEntry.setString(4,entry.getTitle());
            newEntry.setString(5,entry.getYear());
            newEntry.setString(6,entry.getExtd());
            newEntry.setString(7,entry.getGenre());
            newEntry.setString(8,entry.getPlayOrder());
            newEntry.setString(9,entry.getDiscId());
            newEntry.setString(10,entry.getAlbum());
            newEntry.setString(11,entry.getArtist());
            newEntry.setString(12,entry.getArchivePath());
            newEntry.setInt(13,quality);
            newEntry.setString(14,entry.qualityExplanation());
            newEntry.setString(15,entry.getEncoding());
            newEntry.setInt(16,entry.getManyUnicode() ? 1 : 0);
            newEntry.setInt(17,entry.getManyQ() ? 1 : 0);
            newEntry.setInt(18,0); // visited
            newEntry.executeUpdate();
            ResultSet rs = fetchMaxId.executeQuery();
            rs.next();
            int entryId = rs.getInt(1);
            // load the titles
            for (int i = 0; i < entry.getTrackCount(); i++) {
                newTitle.clearParameters();
                newTitle.setInt(1,i+1);
                newTitle.setString(2,entry.getTrack(i));
                newTitle.setString(3,entry.getExtt(i));
                newTitle.setInt(4,entryId);
                newTitle.executeUpdate();
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        } finally {
            insLock.unlock();
        }


    }
}
