package com.tagfriendly.relational;

import com.tagfriendly.jdbc.CacheConnector;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.List;
import java.util.ArrayList;

/**
 * Created by: gary at Oct 25, 2007 3:33:19 PM
 */
public class RelationalLoader {

    private Lock conLock = new ReentrantLock();
    private static Lock artistWriteLock = new ReentrantLock();
    private static Lock albumWriteLock = new ReentrantLock();

    private PreparedStatement getEntry;
    private PreparedStatement getArtist;
    private PreparedStatement insArtist;
    private PreparedStatement insAlbum;
    private PreparedStatement getAlbumId;
    private PreparedStatement insTrack;
    private PreparedStatement getTitles;
    private PreparedStatement setVisited;
    private PreparedStatement getEntries;

    public RelationalLoader(Connection con) throws SQLException {
        new RelationalTableCreation(con).makeTables();
        new RelationalTableCreation(con).makeIndices();
        getEntry = con.prepareStatement("select * from Freedb.Entry where entryId = ?");
        getArtist = con.prepareStatement("select * from Rela.Artist where name = ?");
        insArtist = con.prepareStatement("insert into Rela.Artist(name) values(?)");
        insAlbum = con.prepareStatement("insert into Rela.Album(name,year,genre,discId,artistId,quality) values(?,?,?,?,?,?)");
        getAlbumId = con.prepareStatement("select max(albumId) from Rela.Album where artistId = ? and name=?");
        insTrack = con.prepareStatement("insert into Rela.Track(name,extt,playOrder,albumId) values(?,?,?,?)");
        getTitles = con.prepareStatement("select name,extt,trackOrder from Freedb.Title where entryId = ? order by trackOrder asc");
        setVisited = con.prepareStatement("update Freedb.Entry set visited = 1 where entryId = ?");
        getEntries = con.prepareStatement("select entryId from Freedb.Entry where visited = 0");
        //getEntries = con.prepareStatement("select entryId from Freedb.Entry");
    }

    public List<Integer> getEntriesToProcess() {
        List<Integer> list = new ArrayList<Integer>();
        lock(conLock);
        try {
            ResultSet rs = getEntries.executeQuery();
            try {
            while (rs.next())
                list.add(rs.getInt(1));
            } catch (SQLException ex) {
                throw ex;
            } finally {
                rs.close();
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        } finally {
            unlock(conLock);
        }
        return list;
    }

    public void load(int entryId) throws SQLException {
        // no locking needed because each part is discrete.
        
        // get the free db entry.
        Entry entry = getEntry(entryId);
        if (entry == null) return; // nothing to do.
        // get the artist.
        Artist artist = getArtist(entry.getArtist());
        // save the artist if it is new.
        if (artist.isNew())
            saveArtist(artist);
        // save the album. safe to assume the album does not exist.
        Album album = saveAlbum(artist.getArtistId(),entry);
        // save the tracks off the album.
        saveTracks(entryId,album.getAlbumId());
        //setVisited(entryId);
    }

    private void setVisited(int entryId) throws SQLException {
        lock(conLock);
        try {
            setVisited.clearParameters();
            setVisited.setInt(1,entryId);
            setVisited.executeUpdate();
        } finally {
            unlock(conLock);
        }
    }

    private void saveTracks(int entryId, int albumId) throws SQLException {
        lock(conLock);
        try {
            getTitles.clearParameters();
            getTitles.setInt(1,entryId);
            ResultSet titleRs = getTitles.executeQuery();
            // 1-name, 2-ext, 3-trackOrder,
            List<Track> tracks = new ArrayList<Track>(25);
            while (titleRs.next()) {
                Track tr = new Track(titleRs.getString(1),titleRs.getString(2),titleRs.getInt(3));
                tracks.add(tr);
            }
            titleRs.close();

            for (Track tr : tracks) {
                insTrack.clearParameters();
                insTrack.setString(1,tr.getName());
                insTrack.setString(2,tr.getExtt());
                insTrack.setInt(3,tr.getPlayOrder());
                insTrack.setInt(4,albumId);
                insTrack.executeUpdate();
            }
        } finally {
            unlock(conLock);
        }
    }


    private Album saveAlbum(int artistId, Entry entry) throws SQLException {
        Album album = new Album(entry.getAlbum(),entry.getYear(),entry.getGenre(),entry.getDiscId(),entry.getQuality());
        album.setArtistId(artistId);
        lock(albumWriteLock,conLock);
        // save album, set album id.
        try {
            insAlbum.clearParameters();
            insAlbum.setString(1,album.getName());
            insAlbum.setString(2,album.getYear());
            insAlbum.setString(3,album.getGenre());
            insAlbum.setString(4,album.getDiscId());
            insAlbum.setInt(5,album.getArtistId());
            insAlbum.setInt(6,album.getQuality());
            insAlbum.executeUpdate();
            album.setAlbumId(getAlbumId(artistId,album.getName()));
        } finally {
            unlock(conLock,albumWriteLock);
        }
        return album;
    }

    private int getAlbumId(int artistId, String albumName) throws SQLException {
        int albumId = -1;
        lock(conLock);
        try {
            getAlbumId.clearParameters();
            getAlbumId.setInt(1,artistId);
            getAlbumId.setString(2,albumName);
            ResultSet rs = getAlbumId.executeQuery();
            try {
            if (rs.next())
                albumId = rs.getInt(1);
            } finally {
                rs.close();
            }
        } finally {
            unlock(conLock);
        }
        return albumId;
    }

    private void saveArtist(Artist artist) throws SQLException {
        if (!artist.isNew()) return;
        lock(artistWriteLock,conLock);
        try {
            insArtist.clearParameters();
            insArtist.setString(1,artist.getName());
            insArtist.executeUpdate();
            Artist ar = getArtist(artist.getName());
            artist.setArtistId(ar.getArtistId());
        } finally {
            unlock(conLock,artistWriteLock);
        }
    }

    private Artist getArtist(String name) throws SQLException {
        Artist artist = new Artist(name);
        lock(conLock);
        try {
            getArtist.clearParameters();
            getArtist.setString(1,name);
            ResultSet rs = getArtist.executeQuery();
            try {
                if (rs.next())
                    artist.setArtistId(rs.getInt("artistId"));
            } catch (SQLException ex) {
                throw ex;
            } finally {
                rs.close();
            }
        } finally {
            unlock(conLock);
        }
        return artist;
    }

    private Entry getEntry(int entryId) throws SQLException {
        Entry entry = null;
        lock(conLock);
        try {
            getEntry.clearParameters();;
            getEntry.setInt(1,entryId);
            ResultSet rs = getEntry.executeQuery();
            try {
                if (rs.next()) {
                    entry = new Entry(rs.getInt("discLength"),
                            rs.getString("year"),
                            rs.getString("genre"),
                            rs.getString("discId"),
                            rs.getString("album"),
                            rs.getString("artist"),
                            rs.getInt("quality"));
                }
            } catch (SQLException ex) {
                throw ex;
            } finally {
                rs.close();
            }
        } catch (SQLException ex) {
            throw ex;
        } finally {
            unlock(conLock);
        }

        return entry;
    }

    // Handy utilities for multi-lock locking.

    private static void lock(Lock... locks) {
        for (int i = 0; i < locks.length; i++)
            locks[i].lock();
    }

    private static void unlock(Lock... locks) {
        for (int i = 0; i < locks.length; i++)
            locks[i].unlock();
    }


}
