package com.tagfriendly.solr;

import java.sql.Connection;
import java.sql.SQLException;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: gary
 * Date: Nov 23, 2007
 * Time: 8:21:26 PM
 * To change this template use File | Settings | File Templates.
 */
public class Builder {
    private final BuildWorker[] builder;
    final SolrClient client = SolrClient.newInstance();
    private Thread monitor;
    private Thread optimizer;

    public Builder(Connection... con) throws SQLException {
        ArtistIdVendor vendor = new ArtistIdVendor(con[0]);
        builder = new BuildWorker[con.length];
        for (int i = 0; i < con.length; i++)
            builder[i] = new BuildWorker(con[i],vendor);
    }

    public void start() {
        for (BuildWorker worker : builder)
            worker.start();
        monitor = new Thread("Builder Monitor") {
            public void run() {
                boolean done = false;
                while (!done) {
                    try { sleep(1000L); } catch (Exception ex) {}
                    if (isDone()) {
                        done = true;
                        try {
                            synchronized(optimizer) {
                                optimizer.interrupt();
                            }
                        } catch (Throwable th) {
                            th.printStackTrace();
                        }
                    }
                }

            }
        };

        optimizer = new Thread("Index optimizer") {
            long last;
            public void run() {
                last = System.currentTimeMillis();
                while (isDone()) {
                    try { Thread.sleep(1000 * 60 *60 * 6); }
                    catch (InterruptedException ex) {
                        System.out.println("Optimizer woken up.");
                    }
                    try {
                        client.optimize();
                    } catch (IOException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        };

        optimizer.start();
        monitor.start();
    }

    private boolean isDone() {
        for (BuildWorker worker : builder)
            if (!worker.isDone())
                return false;
        return true;
    }
}
