package com.tagfriendly.solr;

import com.tagfriendly.relational.Album;
import com.tagfriendly.relational.Track;
import com.tagfriendly.relational.Artist;

/**
 * Created by: gary at Nov 5, 2007 10:10:52 PM
 */
class Item {
    public static final String ENCODING = "UTF8";

    public enum Type {
        Track(1), Album(2), Artist(3);

        private int id;
        Type(int i) {
            id = i;
        }

        int id() { return id; }

    };

    private final int id;
    private final String name;
    private final Type type;

    private int trackId = 0;
    private int albumId = 0;
    private int artistId = 0;

    private String trackExtt = "";
    private int trackOrder = 0;
    private String trackName = "";

    private String albumYear = "";
    private String albumGenre = "";
    private String albumDiscId = "";
    private String albumName = "";
    private int albumQuality = Integer.MAX_VALUE;

    private String artistName = "";

    static Item make(Track track) {
        Item item = new Item(track.getTrackId(),track.getName(),Type.Track);
        item.setArtistName(track.getAlbum().getArtist().getName());
        item.setAlbumDiscId(track.getAlbum().getDiscId());
        item.setAlbumGenre(track.getAlbum().getGenre());
        item.setAlbumYear(track.getAlbum().getYear());
        item.setAlbumName(track.getAlbum().getName());
        item.setTrackExtt(track.getExtt());
        item.setTrackOrder(track.getPlayOrder());
        item.setTrackName(track.getName());
        item.artistId = track.getAlbum().getArtist().getArtistId();
        item.albumId = track.getAlbum().getAlbumId();
        item.trackId = track.getTrackId();
        item.setAlbumQuality(track.getAlbum().getQuality());
        return item;
    }

    static Item make(Album album) {
        Item item = new Item(album.getAlbumId(),album.getName(),Type.Album);
        item.setArtistName(album.getArtist().getName());
        item.setAlbumYear(album.getYear());
        item.setAlbumGenre(album.getGenre());
        item.setAlbumDiscId(album.getDiscId());
        item.setAlbumName(album.getName());
        item.albumId = album.getAlbumId();
        item.artistId = album.getArtistId();
        item.setAlbumQuality(album.getQuality());
        return item;
    }

    static Item make(Artist artist) {
        Item item = new Item(artist.getArtistId(),artist.getName(),Type.Artist);
        item.setArtistName(artist.getName());
        item.artistId = artist.getArtistId();
        return item;
    }

    private Item(int id, String name, Type type) {
        this.id = id;
        this.name = name;
        this.type = type;
    }

    String toXml() {
        StringBuilder sb = new StringBuilder();
        sb.append("<doc>");
        sb.append(makeField("unikey",type.id() + ":" + id));
        sb.append(makeField("id",id));
        sb.append(makeField("name",name));
        sb.append(makeField("infoType",type.id()));
        sb.append(makeField("trackId",trackId));
        sb.append(makeField("albumId",albumId));
        sb.append(makeField("artistId",artistId));
        sb.append(makeField("trackExtt",trackExtt));
        sb.append(makeField("trackOrder",trackOrder));
        sb.append(makeField("trackName",trackName));
        sb.append(makeField("albumYear",albumYear));
        sb.append(makeField("albumGenre",albumGenre));
        sb.append(makeField("albumDiscId",albumDiscId));
        sb.append(makeField("albumName",albumName));
        sb.append(makeField("artistName",artistName));
        sb.append(makeField("albumQuality",albumQuality));
        sb.append("</doc>");
        return sb.toString();
    }

    private static String makeField(String name, Object value) {
        return "<field name=\"" + name + "\">" + makeEscapes(value.toString()) + "</field>";
    }

    private static String makeEscapes(String s) {
        s = s.replace("<","&lt;");
        s = s.replace(">","&gt;");
        s = s.replace("&","&amp;");
        s = s.replace("'","&apos;");
        s = s.replace("\"","&quot;");
        return s;
    }

    public void setTrackExtt(String trackExtt) {
        this.trackExtt = trackExtt;
    }

    public void setTrackOrder(int trackOrder) {
        this.trackOrder = trackOrder;
    }

    public void setAlbumYear(String albumYear) {
        this.albumYear = albumYear;
    }

    public void setAlbumGenre(String albumGenre) {
        this.albumGenre = albumGenre;
    }

    public void setAlbumDiscId(String albumDiscId) {
        this.albumDiscId = albumDiscId;
    }

    public void setArtistName(String artistName) {
        this.artistName = artistName;
    }

    public void setTrackName(String trackName) {
        this.trackName = trackName;
    }

    public void setAlbumName(String albumName) {
        this.albumName = albumName;
    }

    public void setAlbumQuality(int quality) {
        this.albumQuality = quality;
    }
}
